@@ -17,11 +17,14 @@
 
 from __future__ import division, print_function, absolute_import
 
+import sys
 from enum import IntEnum
 
-from hypothesis.errors import Frozen, InvalidArgument
+from hypothesis.errors import Frozen, StopTest, InvalidArgument
 from hypothesis.internal.compat import hbytes, hrange, text_type, \
     bit_length, benchmark_time, int_from_bytes, unicode_safe_repr
+from hypothesis.internal.coverage import IN_COVERAGE_TESTS
+from hypothesis.internal.escalation import mark_for_escalation
 
 
 class Status(IntEnum):
@@ -31,13 +34,6 @@ class Status(IntEnum):
     INTERESTING = 3
 
 
-class StopTest(BaseException):
-
-    def __init__(self, testcounter):
-        super(StopTest, self).__init__(repr(testcounter))
-        self.testcounter = testcounter
-
-
 global_test_counter = 0
 
 
@@ -78,6 +74,8 @@ class ConjectureData(object):
         self.forced_indices = set()
         self.capped_indices = {}
         self.interesting_origin = None
+        self.tags = set()
+        self.draw_times = []
 
     def __assert_not_frozen(self, name):
         if self.frozen:
@@ -85,6 +83,9 @@ class ConjectureData(object):
                 'Cannot call %s on frozen ConjectureData' % (
                     name,))
 
+    def add_tag(self, tag):
+        self.tags.add(tag)
+
     @property
     def depth(self):
         return len(self.interval_stack)
@@ -100,19 +101,43 @@ class ConjectureData(object):
         self.output += value
 
     def draw(self, strategy):
-        if self.depth >= MAX_DEPTH:
-            self.mark_invalid()
-        if strategy.is_empty:
-            self.mark_invalid()
-
         if self.is_find and not strategy.supports_find:
             raise InvalidArgument((
                 'Cannot use strategy %r within a call to find (presumably '
                 'because it would be invalid after the call had ended).'
             ) % (strategy,))
+
+        if strategy.is_empty:
+            self.mark_invalid()
+
+        if self.depth >= MAX_DEPTH:
+            self.mark_invalid()
+
+        if self.depth == 0 and not IN_COVERAGE_TESTS:  # pragma: no cover
+            original_tracer = sys.gettrace()
+            try:
+                sys.settrace(None)
+                return self.__draw(strategy)
+            finally:
+                sys.settrace(original_tracer)
+        else:
+            return self.__draw(strategy)
+
+    def __draw(self, strategy):
+        at_top_level = self.depth == 0
         self.start_example()
         try:
-            return strategy.do_draw(self)
+            if not at_top_level:
+                return strategy.do_draw(self)
+            else:
+                start_time = benchmark_time()
+                try:
+                    return strategy.do_draw(self)
+                except BaseException as e:
+                    mark_for_escalation(e)
+                    raise
+                finally:
+                    self.draw_times.append(benchmark_time() - start_time)
         finally:
             if not self.frozen:
                 self.stop_example()
@@ -144,6 +169,7 @@ class ConjectureData(object):
             return
         self.frozen = True
         self.finish_time = benchmark_time()
+
         # Intervals are sorted as longest first, then by interval start.
         for l in self.intervals_by_level:
             for i in hrange(len(l) - 1):
